@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import atexit
 import json
 import os
 import tempfile
@@ -148,7 +149,7 @@ class OSConfigApplierTestCase(testtools.TestCase):
 
     def setUp(self):
         super(OSConfigApplierTestCase, self).setUp()
-        self.useFixture(fixtures.FakeLogger('os-apply-config'))
+        self.logger = self.useFixture(fixtures.FakeLogger('os-apply-config'))
         self.useFixture(fixtures.NestedTempfile())
 
     def test_install_config(self):
@@ -209,6 +210,18 @@ class OSConfigApplierTestCase(testtools.TestCase):
             apply_config.render_template, template(
                 "/etc/glance/script.conf"), {})
 
+    def test_render_template_bad_template(self):
+        tdir = self.useFixture(fixtures.TempDir())
+        bt_path = os.path.join(tdir.path, 'bad_template')
+        with open(bt_path, 'w') as bt:
+            bt.write("{{#foo}}bar={{bar}}{{/bar}}")
+        e = self.assertRaises(config_exception.ConfigException,
+                              apply_config.render_template,
+                              bt_path, {'foo': [{'bar':
+                                                 'abc'}]})
+        self.assertIn('could not render moustache template', str(e))
+        self.assertIn('Section end tag mismatch', self.logger.output)
+
     def test_render_moustache(self):
         self.assertEqual(apply_config.render_moustache("ab{{x.a}}cd", {
                          "x": {"a": "123"}}), "ab123cd")
@@ -241,3 +254,32 @@ class OSConfigApplierTestCase(testtools.TestCase):
                           apply_config.strip_hash, h, 'a.nonexistent')
         self.assertRaises(config_exception.ConfigException,
                           apply_config.strip_hash, h, 'a.c')
+
+    def test_load_list_from_json(self):
+
+        def mkstemp():
+            fd, path = tempfile.mkstemp()
+            atexit.register(
+                lambda: os.path.exists(path) and os.remove(path))
+            return (fd, path)
+
+        def write_contents(fd, contents):
+            with os.fdopen(fd, 'w') as t:
+                t.write(contents)
+                t.flush()
+
+        fd, path = mkstemp()
+        load_list = apply_config.load_list_from_json
+        err = self.assertRaises(ValueError, load_list, path)
+        self.assertEqual("No JSON object could be decoded", str(err))
+        write_contents(fd, json.dumps(["/tmp/config.json"]))
+        json_obj = load_list(path)
+        self.assertEqual(["/tmp/config.json"], json_obj)
+        os.remove(path)
+        self.assertEqual([], load_list(path))
+
+        fd, path = mkstemp()
+        write_contents(fd, json.dumps({}))
+        err = self.assertRaises(ValueError, load_list, path)
+        self.assertEqual(
+            "No list defined in json file: %s" % path, str(err))
